<?php
/**
 * @author    EndpulsE Software http://www.prestashop.endpulse.com
 * @copyright Copyright (C) 2008 - 2019 EndPulse Software
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @package   Endpulse Module Updating System
 * @version   2.1 March 2012
 */

@ini_set('display_errors', 'off');
//DEBUG ONLY
//@ini_set('error_level', '-1');
//@ini_set('display_errors', 'on');
$CDIR = dirname(__FILE__);
require_once("{$CDIR}/../../config/config.inc.php");

class Enupdate
{

    public $currentVersion;

    public $moduleName;

    public $emailOwner;

    public $websiteAddress;

    public $moduleDir;

    public $language;

    public $cdir;

    public $updateUrl = "https://prestashop.endpulse.com/updates.php";

    public function __construct($moduleName, $emailOwner = '', $updateName = '')
    {

        $cookie = new Cookie('psAdmin');
        if (!function_exists('curl_init')) {
            $this->display("CURL support is needed for updating!");
        }
        if (!class_exists("ZipArchive")) {
            $this->display("ZIP archive support is needed for updating!");
        }
        $this->cdir = dirname(__FILE__);
        $this->ds   = DIRECTORY_SEPARATOR;
        if (stristr($this->cdir, $moduleName) === false) {
            $this->display("Current folder is not correct! Contact support");
        }
        $this->moduleName     = $moduleName;
        $this->emailOwner     = $emailOwner;
        $this->updateName     = $updateName;
        $this->language       = Language::getIsoById((!isset($cookie) or !is_object($cookie)) ? (int)(Configuration::get('PS_LANG_DEFAULT')) : (int)($cookie->id_lang));
        $mod                  = Module::getInstanceByName($this->moduleName);
        $this->currentVersion = $mod->getVersion();
        $this->websiteAddress = 'http://' . $_SERVER['HTTP_HOST'] . __PS_BASE_URI__;
    }

    public function checkVersion()
    {

        return $this->getData("check");
    }

    public function backUp()
    {

        if (!is_dir("{$this->cdir}{$this->ds}backups")) {
            mkdir("{$this->cdir}{$this->ds}backups", "0777");
        }
        if (!is_writable("{$this->cdir}{$this->ds}backups")) {
            $this->display("[Backups] folder is not writable! (check permissions)");
        }
        $niceVersion = str_replace(".", "-", $this->currentVersion);
        $name        = "{$this->cdir}{$this->ds}backups{$this->ds}bck_{$this->moduleName}_{$niceVersion}_" . time() . ".zip";
        if (!$this->zip($this->cdir, $name)) {
            $this->display("There was an error with the Backup archive!");
        }

        return true;
    }

    public function zip($source, $destination)
    {

        if (!extension_loaded('zip') || !file_exists($source)) {
            return false;
        }
        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
            return false;
        }
        $source = str_replace('\\', '/', realpath($source));
        if (is_dir($source) === true) {
            $files = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator($source),
                RecursiveIteratorIterator::SELF_FIRST
            );
            foreach ($files as $file) {
                $file = str_replace('\\', '/', realpath($file));
                if (is_dir($file) === true && stristr($file, "backups") === false && stristr(
                    $file,
                    "latest"
                ) === false) {
                    $zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
                } else {
                    if (is_file($file) === true && stristr($file, "tmp.zip") === false) {
                        $zip->addFromString(str_replace($source . '/', '', $file), Tools::file_get_contents($file));
                    }
                }
            }
        } else {
            if (is_file($source) === true) {
                $zip->addFromString(basename($source), Tools::file_get_contents($source));
            }
        }

        if (!$zip->close()) {
            return false;
        }

        return true;
    }

    public function getNewVersion()
    {

        $return = explode("{[|]}", $this->getData("get"));
        if ($return[1] != '') {
            $tmpfile = "{$this->cdir}{$this->ds}tmp.zip";
            if (!file_put_contents($tmpfile, $return[1])) {
                $this->display("Cannot create temp file (check permissions)!");
            }
        }

        return array("message" => $return[0], "tmp" => $tmpfile);
    }

    public function getData($type)
    {

        $data         = (array)$this;
        $data['type'] = $type;
        $ch           = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->updateUrl);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        $response = curl_exec($ch);
        curl_close($ch);

        return $response;
    }

    public function processUpdate($file)
    {

        $zip = new ZipArchive();
        if ($zip->open($file) !== true) {
            unlink($file);
            $this->display("Could not open update archive (check permissions)!");
        }

        if ($zip->extractTo("{$this->cdir}{$this->ds}latest{$this->ds}") !== true) {
            $this->display("Archive was not extracted! (Contact support)");
        }
        $zip->close();
        unlink($file);
        $files = glob("{$this->cdir}{$this->ds}*");
        foreach ($files as $f) {
            if (stristr($f, "backups") === false && stristr($f, "tmp.zip") === false && stristr(
                $f,
                "latest"
            ) === false) {
                $this->rrmdir($f) or $this->display("File or folder [{$f}] cannot be removed - check permissions!");
            }
        }
        $filesExt = glob("{$this->cdir}{$this->ds}latest{$this->ds}{$this->moduleName}{$this->ds}*");
        foreach ($filesExt as $fx) {
            if (stristr($fx, "backups") === false) {
                rename($fx, str_replace(
                    "latest{$this->ds}{$this->moduleName}{$this->ds}",
                    "",
                    $fx
                )) or $this->display("Archive was not processed! (Contact support)");
            }
        }
        $this->rrmdir("{$this->cdir}{$this->ds}latest");

        return true;
    }

    public function display($what)
    {

        $return = json_encode(array("message" => "<span style='color:red'>ERROR: {$what} </span>"));
        print_r($return);
        die();
    }

    public function rrmdir($path)
    {

        return is_file($path) ? @unlink($path) : array_map(
            array(&$this, 'rrmdir'),
            glob("{$path}{$this->ds}*")
        ) == @rmdir($path);
    }
}

if (Tools::getValue('action') != '' && Tools::getValue('moduleName') != '') {
    $u = new Enupdate(Tools::getValue('moduleName'), Tools::getValue('emailOwner'), Tools::getValue('updateName'));
    switch (Tools::getValue('action')) {
        case "check":
            print_r($u->checkVersion());
            break;
        case "update":
            $data = $u->getNewVersion();
            if ($data['tmp'] != '') {
                if ($u->backUp()) {
                    $u->processUpdate($data['tmp']);
                    echo $data['message'];
                }
            } elseif ($data['message']) {
                echo $data['message'];
            } else {
                $u->display($data);
            }
            break;
        default:
            $u->display("Parameters incorrect!");
            break;
    }
} else {
    $u->display("Parameters incorrect!");
}
